#!/bin/env python
"""
KBBTray Big Brother KDE System Tray
Copyright (C) 2003, 2004, 2005 Russell Valentine 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import sys
import string
from string import strip
import socket
import urllib
import re
from kfile import KFileDialog
from kdeui import KMainWindow, KSystemTray, KPopupMenu, KDialogBase, KIntNumInput, KLineEdit, KAboutApplication, KAction, KToggleAction
from kdecore import KApplication, KIconLoader, KIcon, KAboutData, KCmdLineArgs, i18n, KAudioPlayer, KURL, KShortcut
from HTMLParser import HTMLParser
from qt import QPushButton, QLabel, QCursor, SIGNAL, SLOT, QThread, QPixmap, QColor, Qt, QObject, QHBox, QToolTip, QWidget, QGridLayout, QGroupBox, QString, QCustomEvent, QIconSet, PYSIGNAL
from khtml import KHTMLPart, KHTMLView
from kparts import KParts, createReadOnlyPart, createReadWritePart


#Update Tray event
class TrayUpdateEvent(QCustomEvent):
    def __init__(self, theStatus, detailed_status, date):
        QCustomEvent.__init__(self, 44493)
        self.status = theStatus
        self.dstatus = detailed_status
        self.sdate = date
    def getStatus(self):
        return self.status
    def getDStatus(self):
        return self.dstatus
    def getDate(self):
        return self.sdate


#This checks if anything has changed
class KBBTrayChecker(QThread):
    def __init__(self, parent, *args):
        QThread.__init__(self)
        self.parentTray = parent
        self.doStop = None
        self.refreshConfig()
        self.lastDate = "";
        self.lastPoll = 0;
    def refreshConfig(self):
        config = self.parentTray.parentApp.config()
        config.setGroup("Options")
        self.urlGet = str(config.readEntry("bbURL", ""))
        self.pollTime = config.readNumEntry("pollw", 30)
        self.counter = self.pollTime*60
    def stop(self):
        self.doStop = 1
    def run(self):
        status = []
        while not self.doStop:
            if self.counter < self.pollTime*60:
                self.counter = self.counter+1
                self.sleep(1)
            else:
                self.counter = 0
                #Do check
                color=None
                try:
                    if self.urlGet == None:
                        color="unknown"
                    else:
                        f = urllib.urlopen(self.urlGet+"bb.html")
                        page = f.read()
                        f.close
                        parser = MyHTMLParser()
                        parser.feed(page)
                        parser.close()
                        color = parser.get_color()
                        date = parser.get_date()
                        status = parser.get_status()
                        if self.lastDate != date:
                            self.lastDate = date
                            self.lastPoll = 0;
                        else:
                            self.lastPoll += 1;
                            #If date didn't get updated in 10 min, bb not updating
                            if self.lastPoll >= 600/(self.pollTime*60):
                                status=["Timestamp not updating"]
                                color="unknown";
                except:
                    print "Problem connecting or parsing:"+self.urlGet+"bb.html"
                    color="unknown"
                if color == "green":
                    KApplication.postEvent(self.parentTray, TrayUpdateEvent("ok", status, date))
                elif color == "yellow":
                    KApplication.postEvent(self.parentTray, TrayUpdateEvent("attention", status, date))
                elif color == "red":
                    KApplication.postEvent(self.parentTray, TrayUpdateEvent("trouble", status, date))
                elif color == "purple":
                    KApplication.postEvent(self.parentTray, TrayUpdateEvent("no_report", status, date))
                elif color == "clear":
                    KApplication.postEvent(self.parentTray, TrayUpdateEvent("unavailable", status, date))
                elif color == "blue":
                    KApplication.postEvent(self.parentTray, TrayUpdateEvent("offline", status, date))
                else:
                    KApplication.postEvent(self.parentTray, TrayUpdateEvent("unknown", status, date))


#  To parse the bb page for the color of the title #
#  and to parse for what is wrong.  Assumes there  #
#  are not tables inside of tables.                #
class MyHTMLParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.title_seen = 0
        self.group_block_seen = 0 
        self.table_row_seen = 0
        self.font_seen = 0
        self.data = ""
        self.table_data = ""
        self.font_data = ""
        self.status = []
        self.title_color = ""
        self.title_date = ""

    def handle_starttag(self, tag, attrs):
        #Look for the <TITLE> tag
        if tag == "title":
            self.title_seen = 1
        #Look for the <TR> tag
        if tag == "tr" and self.group_block_seen: 
            self.table_row_seen = 1
        #Look for the <FONT> tag
        if tag == "font" and self.group_block_seen: 
            self.font_seen = 1
        #Look for the <TABLE> tag
        if tag == "table":
            for attribute, value in attrs:        
                if attribute == "summary" and value == "Group Block": 
                    self.group_block_seen = 1        
        #Look for the <IMG> tag
        if tag == "img" and self.table_row_seen:
            for attribute, value in attrs:        
                if attribute == "alt": 
                    color = value.split(":")[1]
                    if color != "green":
                        self.status.append(self.font_data + ":" + value)
    def handle_endtag(self, tag):
        #Close the Title
        if tag == "title":
            title_list = self.data.split(" :")
            self.title_color = title_list[0]
            self.title_date = title_list[1].split("@ ")[1]
            self.title_seen = 0
            self.data = ""
        #If we are in a table row and inside a block then reset the font data to ""
        if tag == "tr" and self.group_block_seen: 
            self.table_row_seen = 0
            self.font_data = ""
            #Hopefully at the end of a group block
        if tag == "table":
            self.group_block_seen = 0
            #No more font entry
        if tag == "font" and self.group_block_seen: 
            self.font_seen = 0
    def handle_data(self,data):
        if data:
            if self.title_seen:
                self.data = self.data + data
            if self.font_seen:
                self.font_data = self.font_data + data
    def get_color(self):
        return self.title_color
    def get_date(self):
        return self.title_date
    def get_status(self):
        return self.status

class KBBConfigureDialog(KDialogBase):
    def __init__(self, parent):
        KDialogBase.__init__(self, parent, None, 1, "Configuration", KDialogBase.Ok|KDialogBase.Cancel, KDialogBase.Ok, 1)
        self.parentTray = parent
        self.appConfig = self.parentTray.parentApp.config()
        #TODO: Add validators
        page = self.makeVBoxMainWidget()
        exURLLabel = QLabel("(ex. http://user:password@bbserver.com:80/)", page)
        urlbox = QHBox(page)
        bbURLLable = QLabel("BB URL:", urlbox)
        self.bbURL = KLineEdit(self.appConfig.readEntry("bbURL", ""), urlbox)
        pollbox = QHBox(page)
        pollLabel1 = QLabel("Poll server every:", pollbox)
        self.pollw = KIntNumInput(self.appConfig.readNumEntry("pollw", 2), pollbox)
        self.pollw.setRange(1, 30)
        pollLabel2 = QLabel("Minutes", pollbox)
        #audio sounds
        audiobox = QGroupBox(4, Qt.Horizontal, i18n("Audio Sounds"), page)
        iconLoader = KIconLoader()
        #green
        greenLabel = QLabel("Green sound:", audiobox)
        self.greenSoundFile = KLineEdit(self.appConfig.readEntry("greenSound", ""), audiobox)
        greenButton = QPushButton(audiobox)
        greenButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(greenButton, SIGNAL("released()"), self.pickFileGreen)
        gtestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(gtestButton, SIGNAL("released()"), self.testGreen)
        #yellow
        yelloLabel = QLabel("Yellow sound:", audiobox)
        self.yellowSoundFile = KLineEdit(self.appConfig.readEntry("yellowSound", ""), audiobox)
        yellowButton = QPushButton(audiobox)
        yellowButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(yellowButton, SIGNAL("released()"), self.pickFileYellow)
        ytestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(ytestButton, SIGNAL("released()"), self.testYellow)
        #red
        redLabel = QLabel("Red sound:", audiobox)
        self.redSoundFile = KLineEdit(self.appConfig.readEntry("redSound", ""), audiobox)
        redButton = QPushButton(audiobox)
        redButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(redButton, SIGNAL("released()"), self.pickFileRed)
        rtestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(rtestButton, SIGNAL("released()"), self.testRed)
        #purple
        purpleLabel = QLabel("Purple sound:", audiobox)
        self.purpleSoundFile = KLineEdit(self.appConfig.readEntry("purpleSound", ""), audiobox)
        purpleButton = QPushButton(audiobox)
        purpleButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(purpleButton, SIGNAL("released()"), self.pickFilePurple)
        ptestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(ptestButton, SIGNAL("released()"), self.testPurple)
        #clear
        clearLabel = QLabel("Clear sound:", audiobox)
        self.clearSoundFile = KLineEdit(self.appConfig.readEntry("clearSound", ""), audiobox)
        clearButton = QPushButton(audiobox)
        clearButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(clearButton, SIGNAL("released()"), self.pickFileClear)
        ctestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(ctestButton, SIGNAL("released()"), self.testClear)
        #blue
        blueLabel = QLabel("Blue sound:", audiobox)
        self.blueSoundFile = KLineEdit(self.appConfig.readEntry("blueSound", ""), audiobox)
        blueButton = QPushButton(audiobox)
        blueButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(blueButton, SIGNAL("released()"), self.pickFileBlue)
        btestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(btestButton, SIGNAL("released()"), self.testBlue)
        #unkown
        unknownLabel = QLabel("Unknown sound:", audiobox)
        self.unknownSoundFile = KLineEdit(self.appConfig.readEntry("unknownSound", ""), audiobox)
        unknownButton = QPushButton(audiobox)
        unknownButton.setPixmap(iconLoader.loadIcon("fileopen", KIcon.Small))
        QObject.connect(unknownButton, SIGNAL("released()"), self.pickFileUnknown)
        utestButton = QPushButton(i18n("Test"), audiobox)
        QObject.connect(utestButton, SIGNAL("released()"), self.testUnknown)
    def testGreen(self):
        KAudioPlayer.play(self.greenSoundFile.text())
    def testYellow(self):
        KAudioPlayer.play(self.yellowSoundFile.text())
    def testRed(self):
        KAudioPlayer.play(self.redSoundFile.text())
    def testPurple(self):
        KAudioPlayer.play(self.purpleSoundFile.text())
    def testClear(self):
        KAudioPlayer.play(self.clearSoundFile.text())
    def testBlue(self):
        KAudioPlayer.play(self.blueSoundFile.text())
    def testUnknown(self):
        KAudioPlayer.play(self.unknownSoundFile.text())
    def pickFile(self, color):
        url=KFileDialog.getOpenURL(None, "*.wav|"+str(i18n("Wav files"))+"\n*.*|"+str(i18n("All files")), self, "Choose File")
        if url.isEmpty() == 0:
            if color == "green":
                self.greenSoundFile.setText(url.path())
            elif color == "yellow":
                self.yellowSoundFile.setText(url.path())
            elif color == "red":
                self.redSoundFile.setText(url.path())
            elif color == "purple":
                self.purpleSoundFile.setText(url.path())
            elif color == "clear":
                self.clearSoundFile.setText(url.path())
            elif color == "blue":
                self.blueSoundFile.setText(url.path())                
            elif color == "unknown":
                self.unknownSoundFile.setText(url.path())                
    def pickFileGreen(self):
        self.pickFile("green")
    def pickFileYellow(self):
        self.pickFile("yellow")
    def pickFileRed(self):
        self.pickFile("red")
    def pickFilePurple(self):
        self.pickFile("purple")
    def pickFileClear(self):
        self.pickFile("clear")
    def pickFileBlue(self):
        self.pickFile("blue")
    def pickFileUnknown(self):
        self.pickFile("unknown")
    def slotOk(self):
        self.appConfig.setGroup("Options")
        self.appConfig.writeEntry("bbURL", self.bbURL.text())
        self.appConfig.writeEntry("pollw", self.pollw.value())
        self.appConfig.writeEntry("greenSound", self.greenSoundFile.text())
        self.appConfig.writeEntry("yellowSound", self.yellowSoundFile.text())
        self.appConfig.writeEntry("redSound", self.redSoundFile.text())
        self.appConfig.writeEntry("purpleSound", self.purpleSoundFile.text())
        self.appConfig.writeEntry("clearSound", self.clearSoundFile.text())
        self.appConfig.writeEntry("blueSound", self.blueSoundFile.text())
        self.appConfig.writeEntry("unknownSound", self.unknownSoundFile.text())
        self.appConfig.sync()
        self.parentTray.updateAudioGet()
        self.parentTray.checker.refreshConfig()
        self.accept()

#class KBBTrayWindow(KMainWindow):
class KBBTrayWindow(KParts.MainWindow):
    def __init__(self):
        KMainWindow.__init__(self)
        
        self.w = KHTMLPart (self, "HTMLPart", self);
        self.w.setJScriptEnabled(False)
        self.w.setJavaEnabled(False)
        self.w.setPluginsEnabled(False)
        self.w.openURL(KURL("about:blank"))

        #Right click menu in browser
        self.bpopup = KPopupMenu(self)
        iconLoader = KIconLoader()
        self.backId=self.bpopup.insertItem(iconLoader.loadIconSet("back", KIcon.Small), "&Back", self.back)
        self.bpopup.setItemEnabled(self.backId, False)
        self.forwardId=self.bpopup.insertItem(iconLoader.loadIconSet("forward", KIcon.Small), "&Forward", self.forward)
        self.bpopup.setItemEnabled(self.forwardId, False)
        self.bpopup.insertItem(iconLoader.loadIconSet("reload", KIcon.Small), "&Reload", self.reload)

        #Holds browser history
        self.history = []
        self.historyAt=0;

        self.setCentralWidget(self.w.view())
        self.w.show()
        #TODO: Probably should save this in config
        self.setGeometry(0, 0, 600, 500)
        #Open a link if they click on it
        QObject.connect(self.w.browserExtension(), SIGNAL("openURLRequestDelayed(const KURL &, const KParts::URLArgs &)"), self.clickLink)
        QObject.connect(self.w, SIGNAL("popupMenu(const QString &, const QPoint &)"), self.popupMenu)

    def clickLink(self, url, args):
        self.gotoLink(url.url())

    def gotoLink(self, url):
        #If already on this page don't add to history
        url=str(url)
        if len(self.history) > 0 and self.history[self.historyAt] != url:
            self.history = self.history[0:self.historyAt+1]
            self.history.append(url)
            self.historyAt = len(self.history)-1
        elif len(self.history) <= 0:
            self.history.append(url)
            self.historyAt = len(self.history)-1
        self.updatePopupItems()
        self.w.openURL(KURL(url))

    def updatePopupItems(self):
        #See if back and forward should be enabled or disabled
        if self.historyAt > 0:
            self.bpopup.setItemEnabled(self.backId, True)
        else:
            self.bpopup.setItemEnabled(self.backId, False)
        if self.historyAt < len(self.history)-1:
            self.bpopup.setItemEnabled(self.forwardId, True)
        else:
            self.bpopup.setItemEnabled(self.forwardId, False)
        
    def update(self, url):
        self.gotoLink(url+"bb2.html")
   
    def back(self):
        if self.historyAt > 0:
            self.w.openURL(KURL(self.history[self.historyAt-1]))
            self.historyAt=self.historyAt-1
            self.updatePopupItems()

    def forward(self):
        if self.historyAt+1 <= len(self.history):
            self.w.openURL(KURL(self.history[self.historyAt+1]))
            self.historyAt = self.historyAt+1
            self.updatePopupItems()

    def reload(self):
        #Don't add to history just reload
        self.w.openURL(KURL(self.history[self.historyAt]))

    def popupMenu(self, url, point):
        self.bpopup.popup(point)

    def queryClose(self):
        self.hide()
        return 0
        
class KBBSystemTray(KSystemTray):
    def __init__(self, window, app):
        KSystemTray.__init__(self, window)
        self.parentApp = app
        self.window = window
        #Setup popup menu
        #self.contextMenu().clear()
        iconLoader = KIconLoader()
        self.contextMenu().insertItem(iconLoader.loadIconSet("configure", KIcon.Small), "&Configure KBBTray", self.openConfigDialog)
        self.contextMenu().insertItem(QIconSet(app.miniIcon()), "&About KBBTray", self.doAboutDialog)
        self.isMuted = False
        self.toggleMute = KToggleAction("M&ute Alert Sounds", KShortcut().null(), self.trayMute, self.actionCollection(), "tray_mute")
        self.toggleMute.plug(self.contextMenu())
        #self.contextMenu().insertItem(self.toggleAction)
        QObject.connect(self, SIGNAL("quitSelected()"), self.parentApp, SLOT("quit()"))
        
        #Create all status icons
        #unkown is couldn't connect, or in process
        self.statusIcons = { "unknown": iconLoader.loadIcon("connect_creating", KIcon.Small), \
                             "ok": QPixmap("/usr/share/kbbtray/green.png"), \
                             "attention": QPixmap("/usr/share/kbbtray/yellow.png"), \
                             "trouble": QPixmap("/usr/share/kbbtray/red.png"), \
                             "no_report": QPixmap("/usr/share/kbbtray/purple.png"), \
                             "unavailable": QPixmap("/usr/share/kbbtray/clear.png"), \
                             "offline": QPixmap("/usr/share/kbbtray/blue.png")}
        
        #Audios
        self.audio = { "unknown": None, "ok": None, "attention": None, "trouble": None, "no_report": None, "unavailable": None, "offline": None}
        self.updateAudioGet()
        
        self.status = None
        self.date = None
        self.dStatus = []
        self.setStatus("unknown", self.dStatus, "unknown")
        #Last after icons made start thread checker
        self.checker=KBBTrayChecker(self)
        self.checker.start()

    def customEvent(self, event):
        if event.type() == 44493:
            #event = TrayUpdateEvent(event)
            self.setStatus(event.getStatus(), event.getDStatus(), event.getDate())
            return

    def updateAudioGet(self):
        appConfig = self.parentApp.config()
        appConfig.setGroup("Options")
        self.urlGet = str(appConfig.readEntry("bbURL", ""))
        self.audio = {"unknown": KAudioPlayer(QString(appConfig.readEntry("unknownSound", "")), self), \
                      "ok": KAudioPlayer(QString(appConfig.readEntry("greenSound", "")), self), \
                      "attention": KAudioPlayer(QString(appConfig.readEntry("yellowSound", "")), self), \
                      "trouble": KAudioPlayer(QString(appConfig.readEntry("redSound", "")), self), \
                      "no_report": KAudioPlayer(QString(appConfig.readEntry("purpleSound", "")), self), \
                      "unavailable": KAudioPlayer(QString(appConfig.readEntry("clearSound", "")), self), \
                      "offline": KAudioPlayer(QString(appConfig.readEntry("blueSound", "")), self)}
    
    def __del__(self):
        if self.checker.running() : self.checker.stop()
    def setStatus(self, theStatus, detailed_status, date):
        tooltip = ""
        update = False 
        colorUpdate = False
        dateUpdate = False 
        if self.status != theStatus or len(detailed_status) != len(self.dStatus):
            update = True 
            colorUpdate = True 
        elif self.date != date:
            #TODO: Check if too long period past since last date
            dateUpdate = True 
        else:
            for i in range(len(detailed_status)):
                if detailed_status[i] != self.dStatus[i]:
                    update = True 
                    colorUpdate = True 
                    break
        if update or dateUpdate:
            self.window.update(self.urlGet)
            self.date = date
        if update:
            self.status = theStatus
            self.dStatus = detailed_status
            for i in range(len(self.dStatus)):
                tooltip = tooltip + self.dStatus[i] 
                if i != len(self.dStatus)-1:
                    tooltip = tooltip+"\n"
            if tooltip == "":
                tooltip = "All Green"
            QToolTip.add(self, tooltip)
            if colorUpdate:
#                print "update icon"
                #Change icon here
                if self.status in ("attention", "trouble", "no_report"):
#                    self.setMovie(self.statusIcons[self.status])
                    self.setPixmap(self.statusIcons[self.status])
                else:
                    self.setPixmap(self.statusIcons[self.status])
            #Play color sound
            if self.audio[self.status] != None and not self.isMuted:
#                print "play sound"
                self.audio[self.status].play()
    def openConfigDialog(self):
        configDialog = KBBConfigureDialog(self)
        configDialog.show()
    def doAboutDialog(self):
        theAboutDialog = KAboutApplication(self)
        theAboutDialog.show()
    def trayMute(self):
        if self.toggleMute.isChecked():
            self.isMuted = True
        else:
            self.isMuted = False

def main():
    #Set timeout
    socket.setdefaulttimeout(40)
    aboutData = KAboutData("kbbtray", "KBBTray", "0.07", "A KDE System Tray Big Brother checker", KAboutData.License_GPL, "(c) 2003, 2004, 2005 Russell E. Valentine", None, "http://coldstonelabs.org", "russ@coldstonelabs.org");
    aboutData.addAuthor("Russell Valentine", "Main Programmer", "russ@coldstonelabs.org", "http://users.coldstonelabs.org/russ/");
    aboutData.addAuthor("Michael Garrett", "Programmer/Idea");
    KCmdLineArgs.init(sys.argv, aboutData)
    app=KApplication()
    win=KBBTrayWindow()
    tray=KBBSystemTray(win, app)
    tray.show()
    app.exec_loop()

if __name__=="__main__":
    main()
